#ifndef MYX_REDIS_LEXER_HPP_
#define MYX_REDIS_LEXER_HPP_

#pragma once

#include <myx/base/config.hpp>

#include <QIODevice>

namespace myx {

namespace redis {

class Lexer : public QObject
{
	Q_OBJECT

public:

	explicit Lexer( QIODevice*, QObject* = nullptr );
	~Lexer() override = default;

	Q_SIGNAL void character( char );
	Q_SIGNAL void unsafeString( const QString& );
	Q_SIGNAL void safeString( const QByteArray& );

private:

	Q_SLOT void readData();

	bool readCharacter();
	bool readLength();
	bool readUnsafeString();
	bool readSafeString();

	QIODevice* m_device;
	QByteArray m_buffer;

	enum
	{
		kDoingNothing,
		kReadingLength,
		kReadingUnsafeString,
		kReadingSafeString
	} m_state;

	int m_crlf;
	int m_length;
}; // class Lexer

} // namespace redis

} // namespace myx

#ifdef MYXLIB_HEADER_ONLY
#include "lexer-inl.hpp"
#endif


#endif // ifndef MYX_REDIS_LEXER_HPP_
