#ifndef MYX_REDIS_PARSER_INL_HPP_
#define MYX_REDIS_PARSER_INL_HPP_

#pragma once

#ifndef MYXLIB_HEADER_ONLY
#include <myx/redis/parser.hpp>
#endif

namespace myx {

namespace redis {

Parser::Parser( Lexer* lexer, QObject* parent ) :
	QObject( parent )
{
	connect( lexer, &Lexer::character, this, &Parser::readCharacter );
	connect( lexer, &Lexer::unsafeString, this, &Parser::readUnsafeString );
	connect( lexer, &Lexer::safeString, this, &Parser::readSafeString );
}


void Parser::readCharacter( const char c )
{
	switch ( c )
	{
	case '+':
		m_stack.append( Task( Reply::kStatus ) );    break;
	case '-':
		m_stack.append( Task( Reply::kError ) );     break;
	case ':':
		m_stack.append( Task( Reply::kInteger ) );   break;
	case '$':
		m_stack.append( Task( Reply::kBulk ) );      break;
	case '*':
		m_stack.append( Task( Reply::kMultiBulk ) ); break;
	default:
		break;
	}
}


void Parser::readUnsafeString( const QString& value )
{
	if ( tos().m_reply.type() == Reply::kMultiBulk )
	{
		tos().m_count = value.toInt();
	}
	else
	{
		tos().m_reply.value() = value;
	}

	descend();
}


void Parser::readSafeString( const QByteArray& value )
{
	tos().m_reply.value() = value;
	descend();
}


void Parser::descend()
{
	while ( true )
	{
		if ( ( tos().m_reply.type() == Reply::kMultiBulk ) &&
		     ( tos().m_reply.value().toList().count() < tos().m_count ) )
		{
			return;
		}

		if ( m_stack.count() == 1 )
		{
			auto r = m_stack.takeLast().m_reply;
			Q_EMIT reply( r );
			return;
		}

		auto r = m_stack.takeLast().m_reply;
		tos().m_reply.value().toList().append( QVariant::fromValue( r ) );
	}
}

} // namespace redis

} // namespace myx

#endif // ifndef MYX_REDIS_PARSER_INL_HPP_
